#!/usr/local/bin/python --
## -*-python-*-
##****************************************************************************
##
## File:         dlcoslib.py
## RCS:          $Header: $
## Description:  DL COS access library
## Author:       Scott Hassan, Stanford University
## Created:      
## Modified:     
## Language:     python
## Package:      N/A
## Status:       Experimental (Do Not Distribute)
##
## (c) Copyright 1995, Stanford University, all rights reserved.
##
##*****************************************************************************

## python core modules
import sys, posix, string
import socket
import select

## other modules
import ilu
import ICosProperty

## exceptions
NotFound = "Not Found"
CannotProceed = "Cannot Proceed"
CannotPing = "Cannot Ping Object"


##
## EnableFactoryClient
##
##    This should be that last thing a factory client should do before
##    going into the Ilu.RunMainLoop.
##

def EnableFactoryClient(pObject):
  print pObject.IluSBH()
  print ilu.TypeName(pObject)


def LogMessage(pModule, pMessage, pLevel):
  sys.stderr.write(pModule + ": " + `pMessage` + "\n")

class IOHandler:
  def __init__(self, fd, func, rock):
    self.fd = fd
    self.func = func
    self.rock = rock

  def fileno(self):
    return self.fd

  def call(self):
    self.func(self.fd, self.rock)

class FDMap:
  def __init__(self):
    self.ifdMap = {}
    self.ofdMap = {}

  def RegisterInput(self, iohc):
    fd = iohc.fileno()
    self.ifdMap[fd] = iohc
    return 1

  def RegisterOutput(self, iohc):
    fd = iohc.fileno()
    self.ofdMap[fd] = iohc
    return 1

  def UnregisterInput(self, fd):
    if self.ifdMap.has_key(fd):
      del self.ifdMap[fd]
      return 1
    else:
      return 0

  def UnregisterOutput(self, fd):
    if self.ofdMap.has_key(fd):
      del self.ofdMap[fd]
      return 1
    else:
      return 0

  def ExitMainLoop(self):
    self.notdone = 0

  def RunMainLoop(self):
    self.notdone = 1
    while self.notdone:
##      try:
##      print "inputs: " + `self.ifdMap.keys()`
      results = select.select(self.ifdMap.keys(), self.ofdMap.keys(), [], )
##      except:
##	continue

##      print results
      ifd = results[0]
      ofd = results[1]
      efd = results[2]

      for fd in ifd:
	if self.ifdMap.has_key(fd):
	  self.ifdMap[fd].call()

      for fd in ofd:
	if self.ofdMap.has_key(fd):
	  self.ofdMap[fd].call()

##
## RunMainLoop
##
##

_fdMap = FDMap()

def _reg_inp(iohc):
  global _fdMap
  return _fdMap.RegisterInput(iohc)

def _can_inp(fd):
  global _fdMap
  return _fdMap.UnregisterInput(fd)

def _reg_out(iohc):
  global _fdMap
  return _fdMap.RegisterOutput(iohc)

def _can_out(fd):
  global _fdMap
  return _fdMap.UnregisterOutput(fd)

def _set_alarm(thc):
  pass

def _can_alarm():
  pass

def cbHello(fd, rock):
  global _fdMap
  global _callbacks
##  line = posix.read(fd, 1)
  try:
    line = raw_input()
  except EOFError:
    _fdMap.UnregisterInput(sys.stdin.fileno())
    return
##  try:
  line = string.strip(line)

  tokens = string.split(line)
  if len(tokens) == 0: return

  if tokens[0] == "quit":
    _fdMap.ExitMainLoop()
  elif tokens[0] == "logout":
    _fdMap.ExitMainLoop()
  else:
    if _callbacks.has_key(tokens[0]):
      func, rock = _callbacks[tokens[0]]
      func(line, rock)
##  except:
##    pass

## global
_callbacks = {}

def cbRegisterCommand(pString, pFunction, pRock):
  global _callbacks
  _callbacks[pString] = pFunction, pRock
  

def ExitMainLoop():
  global _fdMap
  _fdMap.ExitMainLoop()

def AddHandler(fd, func, rock):
  global _fdMap
  iohc = IOHandler(fd, func, rock)
  _fdMap.RegisterInput(iohc)
  return iohc

def RemoveHandler(fd):
  global _fdMap
  _fdMap.UnregisterInput(fd)
  
def RunMainLoop(*args):
  global _fdMap
  global _callbacks

  iohc = IOHandler(sys.stdin.fileno(), cbHello, 42)
  _fdMap.RegisterInput(iohc)

  ilu.AddRegisterersToDefault(_reg_inp, _can_inp, _reg_out, _can_out, \
			      _set_alarm, _can_alarm)

  _fdMap.RunMainLoop()

##
## ICosPropertySet Implementation
##

class IPropertySetImpl:
  def __init__(self):
    self.propdb = {}

  ##
  ## property set support methods
  ##

  def define_property(self, property_name, property_value):
    self.propdb[property_name] = property_value

  def define_properties(self, properties):
    for property in properties:
      self.define_property(property['property-name'], property['property-value'])

  def get_number_of_properties(self):
    return len(self.propdb)

  def get_all_property_names(self, how_many):
    keys = self.propdb.keys()
    property_names = keys[:how_many]
    rest = None
    return property_names, rest
    
  def get_property_value(self, property_name):
    if self.propdb.has_key(property_name):
      return self.propdb[property_name]
    else:
      raise ICosProperty.PropertyNotfound

  def get_properties(self, property_names):
    nproperties = []
    ret = ilu.TRUE
    for property_name in property_names:
      if self.propdb.has_key(property_name):
	nproperties.append({"property-name" : property_name, "property-value" : self.propdb[property_name]})
      else:
	ret = ilu.FALSE
	nproperties.append({"property-name" : property_name, "property-value" : None})
      
    return ret, nproperties

  def get_all_properties(self, how_many):
    nproperties = []
    rest = None
    property_names = self.propdb.keys()
    property_names = property_names[:how_many]

    for property_name in property_names:
      nproperties.append({"property-name" : property_name, "property-value" : self.propdb['property_name']})

    return nproperties, rest

  def delete_property(self, property_name):
    if self.propdb.has_key(property_name):
      del self.propdb[property_name]
    else:
      raise ICosProperty.PropertyNotFound

  def delete_properties(self, property_names):
    nproperties = []
    ret = ilu.TRUE
    for property_name in property_names:
      if self.propdb.has_key(property_name):
	del self.propdb[property_name]
      else:
	ret = ilu.FALSE
      
  def delete_all_properties(self):
    ret = ilu.TRUE

    property_names = self.propdb.keys()

    for property_name in property_names:
      del self.propdb[property_name]

    return ret

  def is_property_defined(self, property_name):
    if self.propdb.has_key(property_name):
      return ilu.TRUE
    else:
      return ilu.FALSE

## ----------------------------------------------------------------------

import types

def OutputAny(pAny):
  if type(pAny) == types.IntType:
    return [0, pAny]
  elif type(pAny) == types.LongType:
    return [1, pAny]
  elif type(pAny) == types.FloatType:
    return [4, pAny]
  elif type(pAny) == types.InstanceType:
    return [6, {"aSBH" : pAny.IluSBH(), "aType" : pAny.IluTypeName()}]
  elif type(pAny) == types.StringType:
    return [8, pAny]
  elif type(pAny) == types.ListType:
    anylist = []
    for i in pAny:
      anylist.append(OutputAny(i))
    return [7, anylist]
  else:
    raise TypeError

def InputAny(pAny):
  if pAny[0] == 6:
    return (pAny[1]["aSBH"], pAny[1]["aType"])
  elif pAny[0] == 7:
    anylist = []
    for i in pAny[1]:
      anylist.append(InputAny(i))
    return anylist
  else:
    return pAny[1]
    
  
## ----------------------------------------------------------------------


